﻿using System;
using System.Diagnostics;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using static PROS_Event_Client_DLL.Constants;
using static PROS_Event_Client_Demo.modFunctions;

namespace PROS_Event_Client_Demo
{
    public partial class frmMain : Form
    {
        #region "Variables"
        //Define reference of PROSEventClient
        internal static PROS_Event_Client_DLL.PROSEventClient PROSClient;
        #endregion
        #region "Form Load/Close"
        public frmMain()
        {
            InitializeComponent();
        }
        private void frmMain_Load(object sender, EventArgs e)
        {
            //Create new client
            PROSClient = new PROS_Event_Client_DLL.PROSEventClient();
            //Add event handler for change of the connection status between PROS client and PROS server.
            //Change in the connection state will rise an event, however connection state can be check anytime by checking the 
            //PROSClient.ConnectionState value. Values of the connection states are enumerated with PROS_Event_Client_DLL.Constants.ConnectionStates
            PROSClient.ConnectingStateChange += ConnectionEventHandler;
            //Add event handler for Access control system messages. This handler can be ommited if events are not needed.
            PROSClient.HardwareEvent += HardwareEventHandler;
            //Add handler for Access control system state changes like Door state, Alarm state.
            //Can be ommited if it is not needed
            PROSClient.ChangedDeviceStatus += DeviceStateChangeHandler;
            //Add handler for changes in the Access control system configurations.
            //Can be ommited in most cases.
            PROSClient.ConfigurationChanged += ConfigurationChanged;
        }
        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            //Remove handlers of Client events
            PROSClient.ConnectingStateChange -= ConnectionEventHandler;
            PROSClient.HardwareEvent -= HardwareEventHandler;
            PROSClient.ChangedDeviceStatus -= DeviceStateChangeHandler;
            PROSClient.ConfigurationChanged -= ConfigurationChanged;
            //Ask Client to disconnect from server
            PROSClient.Disconnect();
            PROSClient.Dispose();
            PROSClient = null;
        }
        #endregion
        #region "Form menus"
        /// <summary>
        /// fnConnect(ByVal OneShot as Boolean) will try to connect with PROS server.
        /// If parameter OneShoot is true, Client will try to connect once and return connection status.
        /// If parameter OneShot is false, Client will try to connect until connection is established with the Server or function Disconnect() is called.
        /// </summary>
        private void tsmiConnectOneShot_Click(object sender, EventArgs e)
        {
            if (PROSClient.ConnectionState == (int)ConnectionStates.Connected)
            {
                MessageBox.Show("Already connected!");
                return;
            }
            Form frmConnectX = new frmConnect();
            if (frmConnectX.ShowDialog(this) == DialogResult.OK)
            {
                DisconnectToolStripMenuItem.Enabled = true;
                if (PROSClient.Connect(true) == (int)ConnectionStates.Connected)
                {
                    subDisplayClient();
                    PROSClient.RefreshDevicesStatus();
                    tsslServerVersion.Text = "Server version: " + PROSClient.ServerVersion.ToString();
                }
            }
            frmConnectX.Dispose();
        }
        private void tsmiConnectContinious_Click(object sender, EventArgs e)
        {
            if (PROSClient.ConnectionState == (int)ConnectionStates.Connected)
            {
                MessageBox.Show("Already connected!");
                return;
            }
            Form frmConnectX = new frmConnect();
            if (frmConnectX.ShowDialog(this) == DialogResult.OK)
            {
                DisconnectToolStripMenuItem.Enabled = true;
                if (PROSClient.Connect(false) == (int)ConnectionStates.Connected)
                {
                    subDisplayClient();
                    PROSClient.RefreshDevicesStatus();
                    tsslServerVersion.Text = "Server version: " + PROSClient.ServerVersion.ToString();
                }
            }
            frmConnectX.Dispose();
        }
        /// <summary>
        /// Ask Client to disconnect from Server.
        /// If the Access system Configuration and Users data tables are loaded, they will affected.
        /// </summary>
        private void DisconnectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PROSClient.Disconnect();
        }
        /// <summary>
        /// There are different types of Users in the Access control system.
        /// "Normal User" is user with access permitions.
        /// "Function card" is user that can do various actions in the syste.
        /// Other types are not described here as they are not appearing in the events
        /// </summary>
        private void tsmiUsers_Click(object sender, EventArgs e)
        {
            if (PROSClient.gdtUsers == null)
            {
                MessageBox.Show("Users table is not loaded. You need to connect to server first.");
                return;
            }
            frmUsers frmUsersX = new frmUsers((int)UserTypes.NormalUser);
            frmUsersX.Show(this);
        }
        private void tsmiFunctionCards_Click(object sender, EventArgs e)
        {
            if (PROSClient.gdtUsers == null)
            {
                MessageBox.Show("Users table is not loaded. You need to connect to server first.");
                return;
            }
            frmUsers frmUsersX = new frmUsers((int)UserTypes.FunctionCards);
            frmUsersX.Show(this);
        }
        #endregion
        #region "Hardware treeview Context menus"
        #region "Door"
        /// <summary>
        /// Send commands for Doors
        /// </summary>
        private void tmsiDoorItemOpen_Click(Object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Door, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.DoorReleaseLock);
        }
        private void tsmiDoorItemLock_Click(Object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Door, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.DoorLock);
        }
        private void tsmiDoorItemUnlock_Click(Object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Door, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.DoorUnlock);
        }
        #endregion
        #region "Output"
        /// <summary>
        /// Send comands for Output relays. Door relays are not output relays.
        /// </summary>
        private void tsmiFreeOutputEnable_Click(Object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Output, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.OutputEnable);
        }
        private void tsmiFreeOutputDisable_Click(Object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Output, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.OutputDisable);
        }
        private void tsmiFreeOutputActivate_Click(Object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Output, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.OutputActivate);
        }
        private void tsmiOutputON_Click(object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Output, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.OutputON);
        }
        private void tsmiOutputOFF_Click(object sender, EventArgs e)
        {
            PROSClient.SendCommand((byte)CommandForItem.Output, Convert.ToInt32(tvSystem.SelectedNode.Tag), (byte)ClientRequest.OutputOFF);
        }

        #endregion
        #endregion
        #region "Users treeview Context menus"
        /// <summary>
        /// Select Node forright-click operation
        /// </summary>
        private void tvSystem_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            tvSystem.SelectedNode = e.Node;
        }
        #endregion
        #region "Functions"
        /// <summary>
        /// Read configuration from PROSClient tables and display on treeview
        /// </summary>
        public void subDisplayClient()
        {
            try
            {
                tvSystem.Nodes.Clear();
                foreach (System.Data.DataRow rowPortal in PROSClient.gdtPortals.Rows)
                {
                    tvSystem.Nodes.Add(rowPortal["RedBr"].ToString(), rowPortal["Name"].ToString(), "PortalStateDisconnected", "PortalStateDisconnected");
                    TreeNode tvPortal = tvSystem.Nodes[rowPortal["RedBr"].ToString()];
                    TreeNode tvDoorItems = null;
                    foreach (System.Data.DataRow row in PROSClient.gdtControlers.Select("Portal = " + rowPortal["RedBr"].ToString()))
                    {
                        tvPortal.Nodes.Add("C" + row["RedBr"].ToString(), row["Name"].ToString(), "ControlerStateDisconnected", "ControlerStateDisconnected");
                        TreeNode tvController = tvPortal.Nodes["C" + row["RedBr"].ToString()];
                        foreach (System.Data.DataRow rowDoor in PROSClient.gdtDoors.Select("Controller = " + row["RedBr"].ToString()))
                        {
                            rowDoor["DoorStatus"] = 0;
                            rowDoor["LockStatus"] = 0;
                            rowDoor["Alarm"] = 0;
                            tvController.Nodes.Add("D" + rowDoor["RedBr"].ToString(), rowDoor["Name"].ToString(), "DoorStateUnknown", "DoorStateUnknown");
                            tvController.Nodes["D" + rowDoor["RedBr"].ToString()].ContextMenuStrip = cmsDoorItem;
                            tvController.Nodes["D" + rowDoor["RedBr"].ToString()].Tag = rowDoor["RedBr"].ToString();
                            tvDoorItems = tvController.Nodes["D" + rowDoor["RedBr"].ToString()];
                            tvDoorItems.Nodes.Add("L" + rowDoor["RedBr"].ToString(), rowDoor["Name"].ToString() + " Lock", "LockUnknown", "LockUnknown");
                            tvDoorItems.Nodes.Add("A" + rowDoor["RedBr"].ToString(), rowDoor["Name"].ToString() + "Alarm", "AlarmNormal", "AlarmNormal");
                        }
                        tvController.Nodes.Add("FI", "Inputs", "InputStateON", "InputStateON");
                        foreach (System.Data.DataRow rowFI in PROSClient.gdtFI.Select("Controller = " + row["RedBr"].ToString()))
                        {
                            tvController.Nodes["FI"].Nodes.Add("I" + rowFI["RedBr"].ToString(), rowFI["Name"].ToString(), "InputStateUnknown", "InputStateUnknown");
                        }
                        tvController.Nodes.Add("FO", "Outputs", "OutputStateON", "OutputStateON");
                        foreach (System.Data.DataRow rowFO in PROSClient.gdtFO.Select("Controller = " + row["RedBr"].ToString()))
                        {
                            tvController.Nodes["FO"].Nodes.Add("O" + rowFO["RedBr"].ToString(), rowFO["Name"].ToString(), "OutputStateUnknown", "OutputStateUnknown");
                            tvController.Nodes["FO"].Nodes["O" + rowFO["RedBr"].ToString()].ContextMenuStrip = cmsFreeOutput;
                            tvController.Nodes["FO"].Nodes["O" + rowFO["RedBr"].ToString()].Tag = rowFO["RedBr"].ToString();
                        }
                    }
                }
                tvSystem.ExpandAll();
            }
            catch (Exception ex)
            {
                Debug.Print(DateTime.Now.ToString() + "Error in frmMain/subDisplayClient - " + ex.Message);
            }
        }
        #endregion
        #region "DLL events"
        /// <summary>
        /// Handle connection state changes between PROSClient and PROS Server
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void ConnectionEventHandler(object sender, PROS_Event_Client_DLL.ConnectionStateArgs e)
        {
            dgvEvents.Invoke(new Action(() => { subDisplayConnectionChange(e.ConnectionState); }));
        }
        public void subDisplayConnectionChange(int NewConnectionState)
        {
            tssEventsConnected.Text = Enum.GetName(typeof(ConnectionStates), NewConnectionState);
            dgvEvents.Rows.Insert(0, DateTime.Now.TimeOfDay.ToString(), "PClient.dll", "", "", "", tssEventsConnected.Text, "", "", ilIkoni.Images["DLL"]);
        }
        /// <summary>
        /// Handle hardware events from Access control system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void HardwareEventHandler(object sender, PROS_Event_Client_DLL.HardwareEventArgs e)
        {
            dgvEvents.Invoke(new Action(() => {
                subDisplayEvent(e.EventTime, e.EventsLeft, e.EventID, e.Controller, e.User, e.Door, e.ID, e.Reader, e.FI, e.FO, e.Portal);
            }));
        }
        public void subDisplayEvent(DateTime EventTime, ushort EventsLeft, int EventID, int Controller, int User, int Door, uint ID, int Reader, int FI, int FO, int Portal)
        {
            try
            {
                string EventName = Enum.GetName(typeof(HardwareEvents), EventID);
                string strEventLeft = EventsLeft.ToString();
                string strID = ID.ToString();
                string strEventID = EventID.ToString();
                string PortalName = "";
                if (Portal > 0)
                {
                    DataRow rowPortal = PROSClient.gdtPortals.Rows.Find(Portal);
                    if (rowPortal != null)
                        PortalName = rowPortal["Name"].ToString();
                }
                string ControlerName = "";
                if (Controller > 0)
                {
                    DataRow rowConrtoler = PROSClient.gdtControlers.Rows.Find(Controller);
                    if (rowConrtoler != null)
                        ControlerName = rowConrtoler["Name"].ToString();
                }
                string DoorName = "";
                if (Door > 0)
                {
                    DataRow rowDoor = PROSClient.gdtDoors.Rows.Find(Door);
                    if (rowDoor != null)
                        DoorName = rowDoor["Name"].ToString();
                }
                string ReaderName = "";
                if (Reader > 0)
                {
                    DataRow rowReader = PROSClient.gdtReaders.Rows.Find(Reader);
                    if (rowReader != null)
                        ReaderName = rowReader["Name"].ToString();
                }
                string InputName = "";
                if (FI > 0)
                {
                    DataRow rowFI = PROSClient.gdtFI.Rows.Find(FI);
                    if (rowFI != null)
                        InputName = rowFI["Name"].ToString();
                }
                string OutputName = "";
                if (FO > 0)
                {
                    DataRow rowFO = PROSClient.gdtFO.Rows.Find(FO);
                    if (rowFO != null)
                        OutputName = rowFO["Name"].ToString();
                }
                string UserName = "";
                Image UserImage = ilIkoni.Images["Users.ico"];
                if (User > 0)
                {
                    DataRow rowUser = PROSClient.gdtUsers.Rows.Find(User);
                    if (rowUser != null)
                    {
                        UserName = PROSClient.gdtUsers.Rows.Find(User)["Name"].ToString();
                        if (!(PROSClient.gdtUsers.Rows.Find(User)["Foto"]).Equals(DBNull.Value))
                        {
                            UserImage = fnImageFromBytes((byte[])(PROSClient.gdtUsers.Rows.Find(User)["Foto"]));
                        }
                    }
                }
                if ((EventID == 30) || ((EventID >= 40) && (EventID <= 64)))
                {
                    // user event
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, ReaderName, DoorName, EventName, UserName, strID, UserImage, strEventLeft);
                }
                else if(((EventID >= 128) && (EventID <= 133)) || ((EventID >= 136) && (EventID <= 138)) || ((EventID >= 150) && (EventID <= 154)))
                {
                    //door
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", DoorName, EventName, "", "", ilIkoni.Images["DoorStateClosed"], strEventLeft);
                }
                else if (((EventID >= 134) && (EventID <= 135)) || ((EventID >= 180) && (EventID <= 181)) || (EventID == 205) )
                {
                    //Inputs, urgent warning
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", EventName + " - " + InputName, "", "", ilIkoni.Images["InputStateOFF"], strEventLeft);
                }
                else if (((EventID >= 139) && (EventID <= 143)) || (EventID == 149))
                {
                    //free out
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", EventName + "-" + OutputName, "", "", ilIkoni.Images["OutputStateOFF"],strEventLeft);
                }
                else if ((EventID >= 3) && (EventID <= 5))
                {
                    //function cards
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, ReaderName, "", EventName, UserName, strID, ilIkoni.Images["Reader"],strEventLeft);
                }
                else if (((EventID >= 182) && (EventID <= 183)) || ((EventID >= 201) && (EventID <= 202)))
                {
                    //reader
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, ReaderName, "", EventName, "", "", ilIkoni.Images["Reader"],strEventLeft);
                }
                else if ((EventID == 1) || (EventID == 2) || ((EventID >= 144) && (EventID <= 148)) || (EventID == 158) || 
                    ((EventID >= 175) && (EventID <= 179)) || (EventID == 184) || (EventID == 185) || 
                    ((EventID >= 190) && (EventID <= 193)) || (EventID == 200) || (EventID == 203) || (EventID == 204) || 
                    (EventID == 209) || (EventID == 210) || ((EventID >= 245) && (EventID <= 255)))
                {
                    //Other events
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", EventName, "", "", ilIkoni.Images["ControlerStateConnected"],strEventLeft);
                }
                else
                {
                    //Unknown events
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", strEventID, "", "", ilIkoni.Images["ControlerStateConnected"], strEventLeft);
                }
            }
            catch (Exception ex)
            {
                Debug.Print(DateTime.Now.TimeOfDay.ToString() + "frmMain/subDisplayEvent-Error: " + ex.Message);
            }
        }
        /// <summary>
        /// Handle state changes in the Access control system components
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void DeviceStateChangeHandler(object sender, PROS_Event_Client_DLL.DeviceStatusArgs e)
        {
            dgvEvents.Invoke(new Action(() => { subDisplayDeviceStateChange(e.DeviceType, e.DeviceID, e.Item, e.Status); }));
        }

        public void subDisplayDeviceStateChange(byte DeviceType, int DeviceID, byte Item, int Status)
        {
            switch (DeviceType)
            {
                case (byte)CommandForItem.Portal:
                    switch (Status)
                    {
                        case (byte)ServerResponse.Portal_Offline:
                            tvSystem.Nodes[DeviceID.ToString()].ImageKey = "PortalStateDisconnected";
                            tvSystem.Nodes[DeviceID.ToString()].SelectedImageKey = "PortalStateDisconnected";
                            break;
                        case (byte)ServerResponse.Portal_Online:
                            tvSystem.Nodes[DeviceID.ToString()].ImageKey = "PortalStateConnected";
                            tvSystem.Nodes[DeviceID.ToString()].SelectedImageKey = "PortalStateConnected";
                            break;
                    }
                    break;
                case (byte)CommandForItem.Controller:
                    TreeNode node = tvSystem.Nodes.Find("C" + DeviceID.ToString(), true).First();
                    if ((node != null))
                    {
                        switch (Status)
                        {
                            case (byte)ServerResponse.Controller_Offline:
                                node.ImageKey = "ControlerStateDisconnected";
                                node.SelectedImageKey = "ControlerStateDisconnected";
                                break;
                            case (byte)ServerResponse.Controller_Online:
                                node.ImageKey = "ControlerStateConnected";
                                node.SelectedImageKey = "ControlerStateConnected";
                                break;
                        }
                    }
                    break;
                case (byte)CommandForItem.Door:
                    switch (Item)
                    {
                        case (byte)DoorItems.Door:
                            TreeNode node1 = tvSystem.Nodes.Find("D" + DeviceID.ToString(), true).First();
                            switch (Status)
                            {
                                case (byte)DoorStatus.Closed:
                                    node1.ImageKey = "DoorStateClosed";
                                    node1.SelectedImageKey = "DoorStateClosed";
                                    break;
                                case (byte)DoorStatus.Open:
                                    node1.ImageKey = "DoorStateOpen";
                                    node1.SelectedImageKey = "DoorStateOpen";
                                    break;
                                case (byte)DoorStatus.Unknown:
                                    node1.ImageKey = "DoorStateUnknown";
                                    node1.SelectedImageKey = "DoorStateUnknown";
                                    break;
                            }
                            break;
                        case (byte)DoorItems.Lock:
                            TreeNode node2 = tvSystem.Nodes.Find("L" + DeviceID.ToString(), true).First();
                            switch (Status)
                            {
                                case (byte)LockStatus.Locked:
                                    node2.ImageKey = "LockLocked";
                                    node2.SelectedImageKey = "LockLocked";
                                    break;
                                case (byte)LockStatus.Unlocked:
                                    node2.ImageKey = "LockUnlocked";
                                    node2.SelectedImageKey = "LockUnlocked";
                                    break;
                                case (byte)LockStatus.Unknown:
                                    node2.ImageKey = "LockUnknown";
                                    node2.SelectedImageKey = "LockUnknown";
                                    break;
                                case (byte)LockStatus.Blocked:
                                    node2.ImageKey = "LockBlocked";
                                    node2.SelectedImageKey = "LockBlocked";
                                    break;
                            }
                            break;
                        case (byte)DoorItems.DoorAlarm:
                            TreeNode node3 = tvSystem.Nodes.Find("A" + DeviceID.ToString(), true).First();
                            switch (Status)
                            {
                                case (byte)DoorAlarm.ForcedOpen:
                                    node3.ImageKey = "AlarmDoorForced";
                                    node3.SelectedImageKey = "AlarmDoorForced";
                                    break;
                                case (byte)DoorAlarm.Normal:
                                    node3.ImageKey = "AlarmNormal";
                                    node3.SelectedImageKey = "AlarmNormal";
                                    break;
                                case (byte)DoorAlarm.LeftOpen_Alarm1:
                                    node3.ImageKey = "AlarmLeftOpen";
                                    node3.SelectedImageKey = "AlarmLeftOpen";
                                    break;
                                case (byte)DoorAlarm.OpenTooLong_Alarm2:
                                    node3.ImageKey = "AlarmOpenTooLong";
                                    node3.SelectedImageKey = "AlarmOpenTooLong";
                                    break;
                            }
                            break;
                    }
                    break;
                case (byte)CommandForItem.Input:
                    TreeNode node4 = tvSystem.Nodes.Find("I" + DeviceID.ToString(), true).First();
                    if ((node4 != null))
                    {
                        switch (Status)
                        {
                            case (byte)InputState.InputOFF:
                                node4.ImageKey = "InputStateON";
                                node4.SelectedImageKey = "InputStateON";
                                break;
                            case (byte)InputState.InputON:
                                node4.ImageKey = "InputStateOFF";
                                node4.SelectedImageKey = "InputStateOFF";
                                break;
                            case (byte)InputState.FireON:
                                node4.ImageKey = "FireON";
                                node4.SelectedImageKey = "FireON";
                                break;
                            case (byte)InputState.FireOFF:
                                node4.ImageKey = "FireOFF";
                                node4.SelectedImageKey = "FireOFF";
                                break;
                            case (byte)InputState.Unknown:
                                node4.ImageKey = "InputStateUnknown";
                                node4.SelectedImageKey = "InputStateUnknown";
                                break;
                        }
                    }
                    break;
                case (byte)CommandForItem.Output:
                    TreeNode node5 = tvSystem.Nodes.Find("O" + DeviceID.ToString(), true).First();
                    if ((node5 != null))
                    {
                        switch (Status)
                        {
                            case (byte)OutputState.Disabled:
                                node5.ImageKey = "OutputStateDisabled";
                                node5.SelectedImageKey = "OutputStateDisabled";
                                break;
                            case (byte)OutputState.OutputOFF:
                                node5.ImageKey = "OutputStateOFF";
                                node5.SelectedImageKey = "OutputStateOFF";
                                break;
                            case (byte)OutputState.OutputON:
                                node5.ImageKey = "OutputStateON";
                                node5.SelectedImageKey = "OutputStateON";
                                break;
                            case (byte)OutputState.Unknown:
                                node5.ImageKey = "OutputStateUnknown";
                                node5.SelectedImageKey = "OutputStateUnknown";
                                break;
                        }
                    }
                    break;
                case (byte)CommandForItem.Sistem:
                    switch (Status)
                    {
                        case (byte)ClientRequest.HardwareCommunication_START:
                            tsslHardwarePulling.Text = "Server is pulling for events";
                            break;
                        case (byte)ClientRequest.HardwareCommunication_STOP:
                            tsslHardwarePulling.Text = "Server is NOT pulling for events";
                            break;
                    }
                    break;
            }
        }
        /// <summary>
        /// Handle configuration changes in the Access control system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void ConfigurationChanged(object sender, PROS_Event_Client_DLL.ConfigurationChangedArgs e)
        {
            tvSystem.Invoke(new Action(() => subDisplayConfigurationChange(e.DeviceType, e.Change, e.ItemID)));
        }
        public void subDisplayConfigurationChange(byte DeviceType, byte Change, int ItemID)
        {
            string Message = "";
            switch (DeviceType)
            {
                case (byte)CommandForItem.Portal:
                    Message = "Portal ";
                    break;
                case (byte)CommandForItem.Controller:
                    Message = "Controller ";
                    break;
                case (byte)CommandForItem.Door:
                    Message = "Door ";
                    break;
                case (byte)CommandForItem.Reader:
                    Message = "Reader ";
                    break;
                case (byte)CommandForItem.Input:
                    Message = "Input ";
                    break;
                case (byte)CommandForItem.Output:
                    Message = "Output ";
                    break;
            }
            Message += ItemID.ToString();
            switch (Change)
            {
                case (byte)ServerResponse.Added:
                    Message += " Added";
                    break;
                case (byte)ServerResponse.Edited:
                    Message += " Edited";
                    break;
                case (byte)ServerResponse.Deleted:
                    Message += " Deleted";
                    break;
            }
            dgvEvents.Rows.Insert(0, DateTime.Now, "", "", "", "", Message, "", "", ilIkoni.Images["Server"],
            "");
        }

        #endregion

    }
}
