Imports System.ComponentModel
Imports PROS_Event_Client_DLL.Constants

Public Class frmMain
#Region "Variables"
    'Define reference of PROSEventClient
    Friend PROSClient As PROS_Event_Client_DLL.PROSEventClient

#End Region
#Region "Form Load/Close"
    Private Sub frmMain_Load(sender As Object, e As EventArgs) Handles Me.Load
        'Create new client
        PROSClient = New PROS_Event_Client_DLL.PROSEventClient()
        'Add event handler for change of the connection status between PROS client and PROS server.
        'Change in the connection state will rise an event, however connection state can be check anytime by checking the 
        'PROSClient.ConnectionState value. Values of the connection states are enumerated with PROS_Event_Client_DLL.Constants.ConnectionStates
        AddHandler PROSClient.ConnectingStateChange, AddressOf ConnectionEventHandler
        'Add event handler for Access control system messages. This handler can be ommited if events are not needed.
        AddHandler PROSClient.HardwareEvent, AddressOf HardwareEventHandler
        'Add handler for Access control system state changes like Door state, Alarm state.
        'Can be ommited if it is not needed
        AddHandler PROSClient.ChangedDeviceStatus, AddressOf DeviceStateChangeHandler
        'Add handler for changes in the Access control system configurations.
        'Can be ommited in most cases.
        AddHandler PROSClient.ConfigurationChanged, AddressOf ConfigurationChanged
    End Sub
    Private Sub frmMain_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        'Remove handlers of Client events
        RemoveHandler PROSClient.ConnectingStateChange, AddressOf ConnectionEventHandler
        RemoveHandler PROSClient.HardwareEvent, AddressOf HardwareEventHandler
        RemoveHandler PROSClient.ChangedDeviceStatus, AddressOf DeviceStateChangeHandler
        RemoveHandler PROSClient.ConfigurationChanged, AddressOf ConfigurationChanged
        'Ask Client to disconnect from server
        PROSClient.Disconnect()
        PROSClient.Dispose()
        PROSClient = Nothing
    End Sub

#End Region
#Region "Form menus"
    ''' <summary>
    ''' fnConnect(ByVal OneShot as Boolean) will try to connect with PROS server.
    ''' If parameter OneShoot is true, Client will try to connect once and return connection status.
    ''' If parameter OneShot is false, Client will try to connect until connection is established with the Server or function Disconnect() is called.
    ''' </summary>
    Private Sub tsmiConnectOneShot_Click(sender As Object, e As EventArgs) Handles tsmiConnectOneShot.Click
        If PROSClient.ConnectionState = ConnectionStates.Connected Then
            MsgBox("Already connected!")
            Exit Sub
        End If
        If frmConnect.ShowDialog = DialogResult.OK Then
            DisconnectToolStripMenuItem.Enabled = True
            If PROSClient.Connect(True) = ConnectionStates.Connected Then
                subDisplayClient()
                PROSClient.RefreshDevicesStatus()
                tsslServerVersion.Text = "Server version: " & PROSClient.ServerVersion.ToString
            End If
        End If
    End Sub
    Private Sub tsmiConnectContinious_Click(sender As Object, e As EventArgs) Handles tsmiConnectContinious.Click
        If PROSClient.ConnectionState = ConnectionStates.Connected Then
            MsgBox("Already connected!")
            Exit Sub
        End If
        If frmConnect.ShowDialog = DialogResult.OK Then
            DisconnectToolStripMenuItem.Enabled = True
            If PROSClient.Connect(False) = ConnectionStates.Connected Then
                subDisplayClient()
                PROSClient.RefreshDevicesStatus()
                tsslServerVersion.Text = "Server version: " & PROSClient.ServerVersion.ToString
            End If
        End If
    End Sub
    ''' <summary>
    ''' Ask Client to disconnect from Server.
    ''' If the Access system Configuration and Users data tables are loaded, they will affected.
    ''' </summary>
    Private Sub DisconnectToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles DisconnectToolStripMenuItem.Click
        PROSClient.Disconnect()
    End Sub
    ''' <summary>
    ''' There are different types of Users in the Access control system.
    ''' "Normal User" is user with access permitions.
    ''' "Function card" is user that can do various actions in the syste.
    ''' Other types are not described here as they are not appearing in the events
    ''' </summary>
    Private Sub tsmiUsers_Click(sender As Object, e As EventArgs) Handles tsmiUsers.Click
        If PROSClient.gdtUsers Is Nothing Then
            MsgBox("Users table is not loaded. You need to connect to server first.")
            Exit Sub
        End If
        Dim frmUsersX As New frmUsers(UserTypes.NormalUser)
        frmUsersX.Show(Me)
    End Sub
    Private Sub tsmiFunctionCards_Click(sender As Object, e As EventArgs) Handles tsmiFunctionCards.Click
        If PROSClient.gdtUsers Is Nothing Then
            MsgBox("Users table is not loaded. You need to connect to server first.")
            Exit Sub
        End If
        Dim frmUsersX As New frmUsers(UserTypes.FunctionCards)
        frmUsersX.Show(Me)
    End Sub
#End Region
#Region "Hardware treeview Context menus"
#Region "Door"
    ''' <summary>
    ''' Send commands for Doors
    ''' </summary>
    Private Sub tmsiDoorItemOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDoorItemOpen.Click
        PROSClient.SendCommand(CommandForItem.Door, CInt(tvSystem.SelectedNode.Tag), ClientRequest.DoorReleaseLock)
    End Sub
    Private Sub tsmiDoorItemLock_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDoorItemLock.Click
        PROSClient.SendCommand(CommandForItem.Door, CInt(tvSystem.SelectedNode.Tag), ClientRequest.DoorLock)
    End Sub
    Private Sub tsmiDoorItemUnlock_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDoorItemUnlock.Click
        PROSClient.SendCommand(CommandForItem.Door, CInt(tvSystem.SelectedNode.Tag), ClientRequest.DoorUnlock)
    End Sub
#End Region
#Region "Output"
    ''' <summary>
    ''' Send comands for Output relays. Door relays are not output relays.
    ''' </summary>
    Private Sub tsmiFreeOutputEnable_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFreeOutputEnable.Click
        PROSClient.SendCommand(CommandForItem.Output, CInt(tvSystem.SelectedNode.Tag), ClientRequest.OutputEnable)
    End Sub
    Private Sub tsmiFreeOutputDisable_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFreeOutputDisable.Click
        PROSClient.SendCommand(CommandForItem.Output, CInt(tvSystem.SelectedNode.Tag), ClientRequest.OutputDisable)
    End Sub
    Private Sub tsmiFreeOutputActivate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFreeOutputActivate.Click
        PROSClient.SendCommand(CommandForItem.Output, CInt(tvSystem.SelectedNode.Tag), ClientRequest.OutputActivate)
    End Sub
    Private Sub tsmiOutputON_Click(sender As Object, e As EventArgs) Handles tsmiOutputON.Click
        PROSClient.SendCommand(CommandForItem.Output, CInt(tvSystem.SelectedNode.Tag), ClientRequest.OutputON)
    End Sub
    Private Sub tsmiOutputOFF_Click(sender As Object, e As EventArgs) Handles tsmiOutputOFF.Click
        PROSClient.SendCommand(CommandForItem.Output, CInt(tvSystem.SelectedNode.Tag), ClientRequest.OutputOFF)
    End Sub
#End Region
#End Region
#Region "Users treeview Context menus"
    ''' <summary>
    ''' Select Node forright-click operation
    ''' </summary>
    Private Sub tvSystem_NodeMouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeNodeMouseClickEventArgs) Handles tvSystem.NodeMouseClick
        tvSystem.SelectedNode = e.Node
    End Sub
#End Region
#Region "Functions"
    ''' <summary>
    ''' Read configuration from PROSClient tables and display on treeview
    ''' </summary>
    Sub subDisplayClient()
        Try
            tvSystem.Nodes.Clear()
            For Each rowPortal As Data.DataRow In PROSClient.gdtPortals.Rows
                tvSystem.Nodes.Add(rowPortal("RedBr").ToString, rowPortal("Name").ToString, "PortalStateDisconnected", "PortalStateDisconnected")
                Dim tvPortal As TreeNode = tvSystem.Nodes(rowPortal("RedBr").ToString)
                Dim tvDoorItems As TreeNode
                For Each row As Data.DataRow In PROSClient.gdtControlers.Select("Portal = " & rowPortal("RedBr").ToString)
                    tvPortal.Nodes.Add("C" & row("RedBr").ToString, row("Name").ToString, "ControlerStateDisconnected", "ControlerStateDisconnected")
                    Dim tvController As TreeNode = tvPortal.Nodes("C" & row("RedBr").ToString)
                    For Each rowDoor As Data.DataRow In PROSClient.gdtDoors.Select("Controller = " & row("RedBr").ToString)
                        rowDoor("DoorStatus") = 0
                        rowDoor("LockStatus") = 0
                        rowDoor("Alarm") = 0
                        tvController.Nodes.Add("D" & rowDoor("RedBr").ToString, rowDoor("Name").ToString, "DoorStateUnknown", "DoorStateUnknown")
                        tvController.Nodes("D" & rowDoor("RedBr").ToString).ContextMenuStrip = cmsDoorItem
                        tvController.Nodes("D" & rowDoor("RedBr").ToString).Tag = rowDoor("RedBr").ToString
                        tvDoorItems = tvController.Nodes("D" & rowDoor("RedBr").ToString)
                        tvDoorItems.Nodes.Add("L" & rowDoor("RedBr").ToString, rowDoor("Name").ToString & " Lock", "LockUnknown", "LockUnknown")
                        tvDoorItems.Nodes.Add("A" & rowDoor("RedBr").ToString, rowDoor("Name").ToString & "Alarm", "AlarmNormal", "AlarmNormal")
                    Next
                    tvController.Nodes.Add("FI", "Inputs", "InputStateON", "InputStateON")
                    For Each rowFI As Data.DataRow In PROSClient.gdtFI.Select("Controller = " & row("RedBr").ToString)
                        tvController.Nodes("FI").Nodes.Add("I" & rowFI("RedBr").ToString, rowFI("Name").ToString, "InputStateUnknown", "InputStateUnknown")
                    Next
                    tvController.Nodes.Add("FO", "Outputs", "OutputStateON", "OutputStateON")
                    For Each rowFO As Data.DataRow In PROSClient.gdtFO.Select("Controller = " & row("RedBr").ToString)
                        tvController.Nodes("FO").Nodes.Add("O" & rowFO("RedBr").ToString, rowFO("Name").ToString, "OutputStateUnknown", "OutputStateUnknown")
                        tvController.Nodes("FO").Nodes("O" & rowFO("RedBr").ToString).ContextMenuStrip = cmsFreeOutput
                        tvController.Nodes("FO").Nodes("O" & rowFO("RedBr").ToString).Tag = rowFO("RedBr").ToString
                    Next
                Next
            Next
            tvSystem.ExpandAll()
        Catch ex As Exception
            Debug.Print(Now.ToString & "Error in frmMain/subDisplayClient - " & ex.Message)
        End Try
    End Sub
#End Region
#Region "DLL events"
    ''' <summary>
    ''' Handle connection state changes between PROSClient and PROS Server
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Sub ConnectionEventHandler(sender As Object, e As PROS_Event_Client_DLL.ConnectionStateArgs)
        dgvEvents.Invoke(Sub() subDisplayConnectionChange(e.ConnectionState))
    End Sub
    Sub subDisplayConnectionChange(NewConnectionState As Integer)
        tssEventsConnected.Text = [Enum].GetName(GetType(ConnectionStates), NewConnectionState)
        dgvEvents.Rows.Insert(0, Now.TimeOfDay.ToString, "PClient.dll", "", "", "", tssEventsConnected.Text, "", "", ilIkoni.Images("DLL"))
    End Sub
    ''' <summary>
    ''' Handle hardware events from Access control system
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Sub HardwareEventHandler(sender As Object, e As PROS_Event_Client_DLL.HardwareEventArgs)
        dgvEvents.Invoke(Sub() subDisplayEvent(e.EventTime, e.EventsLeft, e.EventID, e.Controller, e.User, e.Door, e.ID, e.Reader, e.FI, e.FO, e.Portal))
    End Sub
    Sub subDisplayEvent(EventTime As Date, EventsLeft As UShort, EventID As Integer, Controller As Integer, User As Integer, Door As Integer, ID As UInteger, Reader As Integer, FI As Integer, FO As Integer, Portal As Integer)
        Try
            Dim EventName As String = [Enum].GetName(GetType(HardwareEvents), EventID)
            Dim strEventLeft As String = EventsLeft.ToString
            Dim strID As String = ID.ToString
            Dim strEventID As String = EventID.ToString
            Dim PortalName As String = ""
            If Portal > 0 Then
                Dim rowPortal As Data.DataRow = PROSClient.gdtPortals.Rows.Find(Portal)
                If rowPortal IsNot Nothing Then PortalName = rowPortal("Name").ToString
            End If
            Dim ControlerName As String = ""
            If Controller > 0 Then
                Dim rowConrtoler As Data.DataRow = PROSClient.gdtControlers.Rows.Find(Controller)
                If rowConrtoler IsNot Nothing Then ControlerName = rowConrtoler("Name").ToString
            End If
            Dim DoorName As String = ""
            If Door > 0 Then
                Dim rowDoor As Data.DataRow = PROSClient.gdtDoors.Rows.Find(Door)
                If rowDoor IsNot Nothing Then DoorName = rowDoor("Name").ToString
            End If
            Dim ReaderName As String = ""
            If Reader > 0 Then
                Dim rowReader As Data.DataRow = PROSClient.gdtReaders.Rows.Find(Reader)
                If rowReader IsNot Nothing Then ReaderName = rowReader("Name").ToString
            End If
            Dim InputName As String = ""
            If FI > 0 Then
                Dim rowFI As Data.DataRow = PROSClient.gdtFI.Rows.Find(FI)
                If rowFI IsNot Nothing Then InputName = rowFI("Name").ToString
            End If
            Dim OutputName As String = ""
            If FO > 0 Then
                Dim rowFO As Data.DataRow = PROSClient.gdtFO.Rows.Find(FO)
                If rowFO IsNot Nothing Then OutputName = rowFO("Name").ToString
            End If
            Dim UserName As String = ""
            Dim UserImage As Image = ilIkoni.Images("Users.ico")
            If User > 0 Then
                Dim rowUser As Data.DataRow = PROSClient.gdtUsers.Rows.Find(User)
                If rowUser IsNot Nothing Then
                    UserName = PROSClient.gdtUsers.Rows.Find(User)("Name").ToString
                    If Not IsDBNull(PROSClient.gdtUsers.Rows.Find(User)("Foto")) Then
                        UserImage = fnImageFromBytes(CType(PROSClient.gdtUsers.Rows.Find(User)("Foto"), Byte()))
                    End If
                End If
            End If
            Select Case EventID
                Case 30, 40 To 64     'user event
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, ReaderName, DoorName, EventName, UserName, strID, UserImage, strEventLeft)
                Case 128 To  133, 136 To 138, 150 To 154    'door
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", DoorName, EventName, "", "", ilIkoni.Images("DoorStateClosed"), strEventLeft)
                Case 134 To 135, 180 To 181, 205    'Input, Urgent warning
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", EventName & " - " & InputName, "", "", ilIkoni.Images("InputStateOFF"), strEventLeft)
                Case 139 To 143, 149 'free out
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", EventName & "-" & OutputName, "", "", ilIkoni.Images("OutputStateOFF"), strEventLeft)
                Case 3 To 5 'Function cards
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, ReaderName, "", EventName, UserName, strID, ilIkoni.Images("Reader"), strEventLeft)
                Case 182, 183, 201, 202    'reader
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, ReaderName, "", EventName, "", "", ilIkoni.Images("Reader"), strEventLeft)
                Case 1, 2, 158, 175 To 179, 184, 185, 190 To 193, 200, 203, 204, 209, 210, 144 To 148, 245 To 255
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", EventName, "", "", ilIkoni.Images("ControlerStateConnected"), strEventLeft)
                Case Else
                    dgvEvents.Rows.Insert(0, EventTime, PortalName, ControlerName, "", "", strEventID, "", "", ilIkoni.Images("ControlerStateConnected"), strEventLeft)
            End Select
        Catch ex As Exception
            Debug.Print(Now.TimeOfDay.ToString & "frmMain/subDisplayEvent-Error: " & ex.Message)
        End Try
    End Sub
    ''' <summary>
    ''' Handle state changes in the Access control system components
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Sub DeviceStateChangeHandler(sender As Object, e As PROS_Event_Client_DLL.DeviceStatusArgs)
        tvSystem.Invoke(Sub() subDisplayDeviceStateChange(e.DeviceType, e.DeviceID, e.Item, e.Status))
    End Sub
    Sub subDisplayDeviceStateChange(DeviceType As Byte, DeviceID As Integer, Item As Byte, Status As Integer)
        Select Case DeviceType
            Case CommandForItem.Portal
                Select Case Status
                    Case ServerResponse.Portal_Offline
                        tvSystem.Nodes(DeviceID.ToString).ImageKey = "PortalStateDisconnected"
                        tvSystem.Nodes(DeviceID.ToString).SelectedImageKey = "PortalStateDisconnected"
                    Case ServerResponse.Portal_Online
                        tvSystem.Nodes(DeviceID.ToString).ImageKey = "PortalStateConnected"
                        tvSystem.Nodes(DeviceID.ToString).SelectedImageKey = "PortalStateConnected"
                End Select
            Case CommandForItem.Controller
                Dim node As TreeNode = tvSystem.Nodes.Find("C" & DeviceID.ToString, True).First
                If Not node Is Nothing Then
                    Select Case Status
                        Case ServerResponse.Controller_Offline
                            node.ImageKey = "ControlerStateDisconnected"
                            node.SelectedImageKey = "ControlerStateDisconnected"
                        Case ServerResponse.Controller_Online
                            node.ImageKey = "ControlerStateConnected"
                            node.SelectedImageKey = "ControlerStateConnected"
                    End Select
                End If
            Case CommandForItem.Door
                Select Case Item
                    Case DoorItems.Door
                        Dim node As TreeNode = tvSystem.Nodes.Find("D" & DeviceID.ToString, True).First
                        Select Case Status
                            Case DoorStatus.Closed
                                node.ImageKey = "DoorStateClosed"
                                node.SelectedImageKey = "DoorStateClosed"
                            Case DoorStatus.Open
                                node.ImageKey = "DoorStateOpen"
                                node.SelectedImageKey = "DoorStateOpen"
                            Case DoorStatus.Unknown
                                node.ImageKey = "DoorStateUnknown"
                                node.SelectedImageKey = "DoorStateUnknown"
                        End Select
                    Case DoorItems.Lock
                        Dim node As TreeNode = tvSystem.Nodes.Find("L" & DeviceID.ToString, True).First
                        Select Case Status
                            Case LockStatus.Locked
                                node.ImageKey = "LockLocked"
                                node.SelectedImageKey = "LockLocked"
                            Case LockStatus.Unlocked
                                node.ImageKey = "LockUnlocked"
                                node.SelectedImageKey = "LockUnlocked"
                            Case LockStatus.Unknown
                                node.ImageKey = "LockUnknown"
                                node.SelectedImageKey = "LockUnknown"
                            Case LockStatus.Blocked
                                node.ImageKey = "LockBlocked"
                                node.SelectedImageKey = "LockBlocked"
                        End Select
                    Case DoorItems.DoorAlarm
                        Dim node As TreeNode = tvSystem.Nodes.Find("A" & DeviceID.ToString, True).First
                        Select Case Status
                            Case DoorAlarm.ForcedOpen
                                node.ImageKey = "AlarmDoorForced"
                                node.SelectedImageKey = "AlarmDoorForced"
                            Case DoorAlarm.Normal
                                node.ImageKey = "AlarmNormal"
                                node.SelectedImageKey = "AlarmNormal"
                            Case DoorAlarm.LeftOpen_Alarm1
                                node.ImageKey = "AlarmLeftOpen"
                                node.SelectedImageKey = "AlarmLeftOpen"
                            Case DoorAlarm.OpenTooLong_Alarm2
                                node.ImageKey = "AlarmOpenTooLong"
                                node.SelectedImageKey = "AlarmOpenTooLong"
                        End Select
                End Select
            Case CommandForItem.Input
                Dim node As TreeNode = tvSystem.Nodes.Find("I" & DeviceID.ToString, True).First
                If Not node Is Nothing Then
                    Select Case Status
                        Case InputState.InputOFF
                            node.ImageKey = "InputStateON"
                            node.SelectedImageKey = "InputStateON"
                        Case InputState.InputON
                            node.ImageKey = "InputStateOFF"
                            node.SelectedImageKey = "InputStateOFF"
                        Case InputState.FireON
                            node.ImageKey = "FireON"
                            node.SelectedImageKey = "FireON"
                        Case InputState.FireOFF
                            node.ImageKey = "FireOFF"
                            node.SelectedImageKey = "FireOFF"
                        Case InputState.Unknown
                            node.ImageKey = "InputStateUnknown"
                            node.SelectedImageKey = "InputStateUnknown"
                    End Select
                End If
            Case CommandForItem.Output
                Dim node As TreeNode = tvSystem.Nodes.Find("O" & DeviceID.ToString, True).First
                If Not node Is Nothing Then
                    Select Case Status
                        Case OutputState.Disabled
                            node.ImageKey = "OutputStateDisabled"
                            node.SelectedImageKey = "OutputStateDisabled"
                        Case OutputState.OutputOFF
                            node.ImageKey = "OutputStateOFF"
                            node.SelectedImageKey = "OutputStateOFF"
                        Case OutputState.OutputON
                            node.ImageKey = "OutputStateON"
                            node.SelectedImageKey = "OutputStateON"
                        Case OutputState.Unknown
                            node.ImageKey = "OutputStateUnknown"
                            node.SelectedImageKey = "OutputStateUnknown"
                    End Select
                End If
            Case CommandForItem.Sistem
                Select Case Status
                    Case ClientRequest.HardwareCommunication_START
                        tsslHardwarePulling.Text = "Server is pulling for events"
                    Case ClientRequest.HardwareCommunication_STOP
                        tsslHardwarePulling.Text = "Server is NOT pulling for events"
                End Select
        End Select
    End Sub
    ''' <summary>
    ''' Handle configuration changes in the Access control system
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Sub ConfigurationChanged(sender As Object, e As PROS_Event_Client_DLL.ConfigurationChangedArgs)
        tvSystem.Invoke(Sub() subDisplayConfigurationChange(e.DeviceType, e.Change, e.ItemID))
    End Sub
    Sub subDisplayConfigurationChange(DeviceType As Byte, Change As Byte, ItemID As Integer)
        Dim Message As String = ""
        Select Case DeviceType
            Case CommandForItem.Portal
                Message = "Portal "
            Case CommandForItem.Controller
                Message = "Controller "
            Case CommandForItem.Door
                Message = "Door "
            Case CommandForItem.Reader
                Message = "Reader "
            Case CommandForItem.Input
                Message = "Input "
            Case CommandForItem.Output
                Message = "Output "
        End Select
        Message &= ItemID.ToString
        Select Case Change
            Case ServerResponse.Added
                Message &= " Added"
            Case ServerResponse.Edited
                Message &= " Edited"
            Case ServerResponse.Deleted
                Message &= " Deleted"
        End Select
        dgvEvents.Rows.Insert(0, Now, "", "", "", "", Message, "", "", ilIkoni.Images("Server"), "")
    End Sub


#End Region
End Class
